/*
 * Pelion Device Management JavaScript SDK
 * Copyright Arm Limited 2017
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { DeviceStateObserver } from "./observers/deviceStateObserver";
import { MasterObserver } from "./observers/masterObserver";
import { ResourceValuesObserver } from "./observers/resourceValuesObserver";
export class Subscribe {
    constructor(_connect) {
        if (_connect) {
            this.connect = _connect;
        }
        this.deviceStateObservers = new Array();
        this.resourceValueObservers = new Array();
        this.masterObserver = new MasterObserver();
    }
    /**
     * Returns a deviceStateObserver that is listening for device events.
     * To subscribe to specific events or devices, a DeviceEventFilter must be provided.
     *
     * Example: subscribe to device registration events.
     *
     * ```javascript
     * const observer = connect.subscribe.deviceStateChanges({ event: "registrations" });
     * // add a callback
     * observer.addCallback(res => console.log(res));
     * ```
     *
     * @param filter the deviceEventFilter
     */
    deviceStateChanges(filter) {
        const observer = new DeviceStateObserver(filter);
        this.deviceStateObservers.push(observer);
        this.startNotifications();
        return observer;
    }
    resourceValues(filter, immediacy = "OnValueUpdate") {
        return __awaiter(this, void 0, void 0, function* () {
            const observer = new ResourceValuesObserver(filter, this.connect, immediacy);
            yield observer.syncPresubscriptions();
            this.resourceValueObservers.push(observer);
            this.startNotifications();
            return observer;
        });
    }
    /**
     * Returns the master observer that is listening to all notifications coming from Pelion.
     */
    allNotifications() {
        this.startNotifications();
        return this.masterObserver;
    }
    /**
     * Notify all observers
     * @param data
     */
    notifyDeviceEvents(data) {
        this.deviceStateObservers.forEach(observer => observer.notify(data));
    }
    /**
     * Notify all observers
     * @param data
     */
    notifyResourceValues(data) {
        this.resourceValueObservers.forEach(observer => observer.notify(data));
    }
    /**
     * Notify the master observer
     * @param data
     */
    notifyAllNotifications(data) {
        this.masterObserver.notify(data);
    }
    /**
     * Notify all observers
     * @param data
     */
    startNotifications() {
        if (this.connect && this.connect.autostartNotifications) {
            this.connect.startNotifications();
        }
    }
}
//# sourceMappingURL=subscribe.js.map